import pytest

from acore_spellscript_refactor.refactor import *

def test_always_passes():
    assert True

@pytest.mark.parametrize('lines, expect',  [
    ([
        'class spell_yogg_saron_malady_of_the_mind : public SpellScriptLoader',
        '{',
        'public:',
        '    spell_yogg_saron_malady_of_the_mind() : SpellScriptLoader("spell_yogg_saron_malady_of_the_mind_something") { }'
        ], 'spell_yogg_saron_malady_of_the_mind_something')
])

def test_find_name_of_script(lines, expect):
    assert find_name_of_script(lines) == expect


import pytest

from acore_spellscript_refactor.refactor import *

FILENAME_READ = "tests/test_spell_script/spell_script_read.cpp"
FILENAME_WRITE = "tests/test_spell_script/spell_script_write.cpp"
FILENAME_EXPECTED = "tests/test_spell_script/spell_script_expected.cpp"

@pytest.fixture
def expect():
    with open(FILENAME_EXPECTED, 'r') as file:
        lines = file.read();
        return lines.split('\n')

def test_always_passes():
    assert True

def test_format_first_block_in_file(expect):
    format_first_block_in_file(FILENAME_READ, FILENAME_WRITE)

    with open(FILENAME_WRITE, 'r') as file:
        lines = file.read();
        lines = lines.split('\n')

    assert len(lines) == len(expect)
    for i in range(len(lines)):
        assert lines[i] == expect[i], f"\'{i}:\'{lines[i]}\'\'{expect[i]}\'"


import pytest

from acore_spellscript_refactor.refactor import *

input_as_string =\
"""// 63802 - Brain Link
class spell_yogg_saron_brain_link : public SpellScriptLoader
{
public:
    spell_yogg_saron_brain_link() : SpellScriptLoader("spell_yogg_saron_brain_link") { }

    class spell_yogg_saron_brain_link_AuraScript : public AuraScript
    {
        PrepareAuraScript(spell_yogg_saron_brain_link_AuraScript);

        void HandleOnEffectApply(AuraEffect const* /*aurEff*/, AuraEffectHandleModes /*mode*/)
        {
            PreventDefaultAction();
            Player* target = nullptr;
            Map::PlayerList const& pList = GetUnitOwner()->GetMap()->GetPlayers();
            uint8 _offset = urand(0, pList.getSize() - 1);
            uint8 _counter = 0;
            for(Map::PlayerList::const_iterator itr = pList.begin(); itr != pList.end(); ++itr, ++_counter)
            {
                if (itr->GetSource() == GetUnitOwner() || GetUnitOwner()->GetDistance(itr->GetSource()) > 50.0f || !itr->GetSource()->IsAlive() || itr->GetSource()->IsGameMaster())
                    continue;

                if (_counter <= _offset || !target)
                    target = itr->GetSource();
                else
                    break;
            }

            if (!target)
                SetDuration(0);
            else
                _targetGUID = target->GetGUID();
        }

        void OnPeriodic(AuraEffect const*  /*aurEff*/)
        {
            Unit* owner = GetUnitOwner();
            if (!owner)
            {
                SetDuration(0);
                return;
            }

            Unit* _target = ObjectAccessor::GetUnit(*owner, _targetGUID);
            if (!_target || !_target->IsAlive() || std::fabs(owner->GetPositionZ() - _target->GetPositionZ()) > 10.0f) // Target or owner underground
            {
                SetDuration(0);
                return;
            }

            if (owner->GetDistance(_target) > 20.0f)
            {
                owner->CastSpell(_target, SPELL_BRAIN_LINK_DAMAGE, true);
                owner->CastSpell(owner, SPELL_BRAIN_LINK_DAMAGE, true);
            }
            else
                owner->CastSpell(_target, SPELL_BRAIN_LINK_OK, true);
        }

        void Register() override
        {
            OnEffectApply += AuraEffectApplyFn(spell_yogg_saron_brain_link_AuraScript::HandleOnEffectApply, EFFECT_0, SPELL_AURA_PERIODIC_DUMMY, AURA_EFFECT_HANDLE_REAL);
            OnEffectPeriodic += AuraEffectPeriodicFn(spell_yogg_saron_brain_link_AuraScript::OnPeriodic, EFFECT_0, SPELL_AURA_PERIODIC_DUMMY);
        }

    protected:
        ObjectGuid _targetGUID;
    };

    AuraScript* GetAuraScript() const override
    {
        return new spell_yogg_saron_brain_link_AuraScript();
    }

    class spell_yogg_saron_brain_link_SpellScript : public SpellScript
    {
        PrepareSpellScript(spell_yogg_saron_brain_link_SpellScript);

        void FilterTargets(std::list<WorldObject*>& targets)
        {
            std::list<WorldObject*> tempList;
            for (std::list<WorldObject*>::iterator itr = targets.begin(); itr != targets.end(); ++itr)
                if ((*itr)->GetPositionZ() > 300.0f)
                    tempList.push_back(*itr);

            targets.clear();
            for (std::list<WorldObject*>::iterator itr = tempList.begin(); itr != tempList.end(); ++itr)
                targets.push_back(*itr);
        }

        void Register() override
        {
            OnObjectAreaTargetSelect += SpellObjectAreaTargetSelectFn(spell_yogg_saron_brain_link_SpellScript::FilterTargets, EFFECT_0, TARGET_UNIT_SRC_AREA_ENEMY);
        }
    };

    SpellScript* GetSpellScript() const override
    {
        return new spell_yogg_saron_brain_link_SpellScript();
    }
};"""

expected_output_spell =\
"""class spell_yogg_saron_brain_link : public SpellScript
{
    PrepareSpellScript(spell_yogg_saron_brain_link);

    void FilterTargets(std::list<WorldObject*>& targets)
    {
        std::list<WorldObject*> tempList;
        for (std::list<WorldObject*>::iterator itr = targets.begin(); itr != targets.end(); ++itr)
            if ((*itr)->GetPositionZ() > 300.0f)
                tempList.push_back(*itr);

        targets.clear();
        for (std::list<WorldObject*>::iterator itr = tempList.begin(); itr != tempList.end(); ++itr)
            targets.push_back(*itr);
    }

    void Register() override
    {
        OnObjectAreaTargetSelect += SpellObjectAreaTargetSelectFn(spell_yogg_saron_brain_link::FilterTargets, EFFECT_0, TARGET_UNIT_SRC_AREA_ENEMY);
    }
};"""

expected_output_aura =\
"""class spell_yogg_saron_brain_link_aura : public AuraScript
{
    PrepareAuraScript(spell_yogg_saron_brain_link_aura);

    bool Validate(SpellInfo const* /*spellInfo*/) override
    {
        return ValidateSpellInfo({ SPELL_BRAIN_LINK_DAMAGE, SPELL_BRAIN_LINK_OK });
    }

    void HandleOnEffectApply(AuraEffect const* /*aurEff*/, AuraEffectHandleModes /*mode*/)
    {
        PreventDefaultAction();
        Player* target = nullptr;
        Map::PlayerList const& pList = GetUnitOwner()->GetMap()->GetPlayers();
        uint8 _offset = urand(0, pList.getSize() - 1);
        uint8 _counter = 0;
        for(Map::PlayerList::const_iterator itr = pList.begin(); itr != pList.end(); ++itr, ++_counter)
        {
            if (itr->GetSource() == GetUnitOwner() || GetUnitOwner()->GetDistance(itr->GetSource()) > 50.0f || !itr->GetSource()->IsAlive() || itr->GetSource()->IsGameMaster())
                continue;

            if (_counter <= _offset || !target)
                target = itr->GetSource();
            else
                break;
        }

        if (!target)
            SetDuration(0);
        else
            _targetGUID = target->GetGUID();
    }

    void OnPeriodic(AuraEffect const*  /*aurEff*/)
    {
        Unit* owner = GetUnitOwner();
        if (!owner)
        {
            SetDuration(0);
            return;
        }

        Unit* _target = ObjectAccessor::GetUnit(*owner, _targetGUID);
        if (!_target || !_target->IsAlive() || std::fabs(owner->GetPositionZ() - _target->GetPositionZ()) > 10.0f) // Target or owner underground
        {
            SetDuration(0);
            return;
        }

        if (owner->GetDistance(_target) > 20.0f)
        {
            owner->CastSpell(_target, SPELL_BRAIN_LINK_DAMAGE, true);
            owner->CastSpell(owner, SPELL_BRAIN_LINK_DAMAGE, true);
        }
        else
            owner->CastSpell(_target, SPELL_BRAIN_LINK_OK, true);
    }

    void Register() override
    {
        OnEffectApply += AuraEffectApplyFn(spell_yogg_saron_brain_link_aura::HandleOnEffectApply, EFFECT_0, SPELL_AURA_PERIODIC_DUMMY, AURA_EFFECT_HANDLE_REAL);
        OnEffectPeriodic += AuraEffectPeriodicFn(spell_yogg_saron_brain_link_aura::OnPeriodic, EFFECT_0, SPELL_AURA_PERIODIC_DUMMY);
    }

protected:
    ObjectGuid _targetGUID;
};"""

expected_out= expected_output_spell +'\n'+expected_output_aura

@pytest.fixture
def lines():
    return input_as_string.split('\n')

@pytest.fixture
def expect() -> str:
    return expected_out

@pytest.fixture
def expect_aura() -> str:
    return expected_output_aura.split('\n')

@pytest.fixture
def expect_spell() -> str:
    return expected_output_spell.split('\n')

@pytest.fixture
def expect_spell_string() -> str:
    return "RegisterSpellAndAuraScriptPair(spell_yogg_saron_brain_link, spell_yogg_saron_brain_link_aura);"

def test_always_passes():
    assert True

def test_find_start_last_index(lines):
    start_index, last_index = find_start_last_index(lines)
    assert start_index == 1
    assert last_index == 100

def test_find_name_of_script(lines):
    assert find_name_of_script(lines) == "spell_yogg_saron_brain_link"

def test_get_script_type(lines):
    start_index, last_index = find_start_last_index(lines)
    assert get_script_type(lines, start_index) == ScriptType.PAIR

def test_find_spell_block_in_pair(lines, expect_spell):
    start_index, last_index = find_start_last_index(lines)
    spell_script_lines = find_spell_block_in_pair(lines[start_index:last_index], ScriptType.SPELL)
    assert len(expect_spell) == len(spell_script_lines)

def test_convert_function_block(lines, expect, expect_spell, expect_aura):
    converted_pair, spell_type, start_index, last_index, _, _ = convert_function_block(lines)
    assert spell_type == ScriptType.PAIR

    spell, aura = converted_pair
    spell, aura = spell.split('\n'), aura.split('\n')

    assert len(expect_spell) == len(spell)
    for i in range(len(aura)):
        assert aura[i] == expect_aura[i], f"{i-1}:\'{aura[i-1]}\'{i}:\'{aura[i]}\'\'{expect_aura[i]}\'"
    assert len(expect_aura) == len(aura)
    assert len(expect_aura) == len(aura)
    for i in range(len(spell)):
        assert spell[i] == expect_spell[i], f"{i-1}:\'{spell[i-1]}\'{i}:\'{spell[i]}\'\'{expect_spell[i]}\'"


from enum import Enum

class Color(Enum):
    RED = 1
    GREEN = 2
    BLUE = 3
    YELLOW = 4

def color(text, color, end='\n') -> str:
    colors = {Color.RED: '\x1b[31m', Color.GREEN: '\x1b[32m', Color.YELLOW: '\x1b[33m', Color.BLUE: '\x1b[34m'}
    reset = '\x1b[0m'
    return colors.get(color, '') + text + reset + end


import pytest

from acore_spellscript_refactor.refactor import *

FILENAME = "tests/test_with_input_file/boss_yoggsaron.cpp"

@pytest.fixture
def lines():
    with open(FILENAME, 'r') as file:
        lines = file.read();
        return lines.split('\n')

@pytest.fixture
def expect() -> str:
    return expected_output

@pytest.fixture
def expect_spell_string() -> str:
    return "RegisterSpellScript(spell_yogg_saron_malady_of_the_mind_aura);"

expected_output =\
"""class spell_yogg_saron_malady_of_the_mind_aura : public AuraScript
{
    PrepareAuraScript(spell_yogg_saron_malady_of_the_mind_aura);

    bool Validate(SpellInfo const* /*spellInfo*/) override
    {
        return ValidateSpellInfo({ SPELL_DEATH_RAY_DAMAGE_REAL, SPELL_MALADY_OF_THE_MIND_TRIGGER });
    }

    void OnApply(AuraEffect const* /*aurEff*/, AuraEffectHandleModes /*mode*/)
    {
        GetUnitOwner()->ApplySpellImmune(SPELL_DEATH_RAY_DAMAGE_REAL, IMMUNITY_ID, SPELL_DEATH_RAY_DAMAGE_REAL, true);
    }

    void OnRemove(AuraEffect const* /*aurEff*/, AuraEffectHandleModes /*mode*/)
    {
        GetUnitOwner()->ApplySpellImmune(SPELL_DEATH_RAY_DAMAGE_REAL, IMMUNITY_ID, SPELL_DEATH_RAY_DAMAGE_REAL, false);
        GetUnitOwner()->CastCustomSpell(SPELL_MALADY_OF_THE_MIND_TRIGGER, SPELLVALUE_MAX_TARGETS, 1, GetUnitOwner(), true);
    }

    void Register() override
    {
        OnEffectApply += AuraEffectApplyFn(spell_yogg_saron_malady_of_the_mind_aura::OnApply, EFFECT_1, SPELL_AURA_MOD_FEAR, AURA_EFFECT_HANDLE_REAL);
        OnEffectRemove += AuraEffectRemoveFn(spell_yogg_saron_malady_of_the_mind_aura::OnRemove, EFFECT_1, SPELL_AURA_MOD_FEAR, AURA_EFFECT_HANDLE_REAL);
    }
};"""

def test_always_passes():
    assert True

def test_find_start_last_index(lines):
    start_index, last_index = find_start_last_index(lines)
    assert start_index == 2234, lines[start_index]
    assert last_index == 2265

def test_find_name_of_script(lines):
    start_index, last_index = find_start_last_index(lines)
    assert find_name_of_script(lines, start_index) == "spell_yogg_saron_malady_of_the_mind"

def test_get_script_type(lines):
    start_index, last_index = find_start_last_index(lines)
    assert get_script_type(lines, start_index) == ScriptType.AURA

def test_find_register_start_end_index(lines):
    start_index, last_index = find_start_last_index(lines)
    register_start_index, register_end_index = find_register_start_end_index(lines, start_index)
    assert lines[register_start_index].lstrip() == "void Register() override"
    assert register_end_index == register_start_index + 4
    assert lines[register_end_index] == "        }"

def test_find_register_statements(lines):
    register_statements = find_register_statements(lines, 1)
    assert len(register_statements) == 2
    assert register_statements[0] == "            OnEffectApply += AuraEffectApplyFn(spell_yogg_saron_malady_of_the_mind_AuraScript::OnApply, EFFECT_1, SPELL_AURA_MOD_FEAR, AURA_EFFECT_HANDLE_REAL);"

def test_format_register_statements():
    register_statements = ["            OnEffectProc += AuraEffectProcFn(spell_yogg_saron_grim_reprisal_AuraScript::HandleProc, EFFECT_0, SPELL_AURA_DUMMY);"]
    formatted_statements = format_register_statements(register_statements)
    assert len(formatted_statements) == 1
    assert formatted_statements[0] == "        OnEffectProc += AuraEffectProcFn(spell_yogg_saron_grim_reprisal_aura::HandleProc, EFFECT_0, SPELL_AURA_DUMMY);"

def test_find_content_start_end_index(lines):
    start_index, last_index = find_start_last_index(lines)
    content_index_start, content_index_end = find_content_start_end_index(lines, start_index)
    assert lines[content_index_start].lstrip() == "PrepareAuraScript(spell_yogg_saron_malady_of_the_mind_AuraScript);"
    assert lines[content_index_end].lstrip() == "void Register() override"

def test_format_content_start_end_index(lines):
    content_index_start, content_index_end = find_content_start_end_index(lines)
    content_statements = lines[content_index_start+1:content_index_end]
    formatted_content_statements = format_content_statements(content_statements);
    assert len(formatted_content_statements) == 12
    assert formatted_content_statements[1] == "    void OnApply(AuraEffect const* /*aurEff*/, AuraEffectHandleModes /*mode*/)"
    assert formatted_content_statements[2] == "    {"
    assert all(f.strip() != '' or (f.strip() == '' and f == '') for f in formatted_content_statements)

def test_convert_function_block(lines, expect, expect_spell_string):
    out, spell_type, start_index, last_index, _, _ = convert_function_block(lines)
    assert spell_type == ScriptType.AURA
    assert last_index - start_index == 31
    assert expect == out
    out = out.split('\n')
    expect = expect.split('\n')
    assert len(expect) == len(out)
    for i in range(len(out)):
        assert out[i] == expect[i], f"{i-1}:\'{out[i-1]}\'{i}:\'{out[i]}\'\'{expect[i]}\'"



import pytest

from acore_spellscript_refactor.refactor import *

# pytest --pyargs spellscript_refactor
input_as_string =\
"""// 63305 - Grim Reprisal
class spell_yogg_saron_grim_reprisal : public SpellScriptLoader
{
public:
    spell_yogg_saron_grim_reprisal() : SpellScriptLoader("spell_yogg_saron_grim_reprisal") { }

    class spell_yogg_saron_grim_reprisal_AuraScript : public AuraScript
    {
        PrepareAuraScript(spell_yogg_saron_grim_reprisal_AuraScript);

        bool Validate(SpellInfo const* /*spellInfo*/) override
        {
            return ValidateSpellInfo({ SPELL_GRIM_REPRISAL_DAMAGE });
        }

        void HandleProc(AuraEffect const* aurEff, ProcEventInfo& eventInfo)
        {
            DamageInfo* damageInfo = eventInfo.GetDamageInfo();

            if (!damageInfo || !damageInfo->GetDamage())
            {
                return;
            }

            int32 damage = CalculatePct(static_cast<int32>(damageInfo->GetDamage()), 60);
        }

        void Register() override
        {
            OnEffectProc += AuraEffectProcFn(spell_yogg_saron_grim_reprisal_AuraScript::HandleProc, EFFECT_0, SPELL_AURA_DUMMY);
        }
    };

    AuraScript* GetAuraScript() const override
    {
        return new spell_yogg_saron_grim_reprisal_AuraScript();
    }
};
"""

expected_output =\
"""class spell_yogg_saron_grim_reprisal_aura : public AuraScript
{
    PrepareAuraScript(spell_yogg_saron_grim_reprisal_aura);

    bool Validate(SpellInfo const* /*spellInfo*/) override
    {
        return ValidateSpellInfo({ SPELL_GRIM_REPRISAL_DAMAGE });
    }

    void HandleProc(AuraEffect const* aurEff, ProcEventInfo& eventInfo)
    {
        DamageInfo* damageInfo = eventInfo.GetDamageInfo();

        if (!damageInfo || !damageInfo->GetDamage())
        {
            return;
        }

        int32 damage = CalculatePct(static_cast<int32>(damageInfo->GetDamage()), 60);
    }

    void Register() override
    {
        OnEffectProc += AuraEffectProcFn(spell_yogg_saron_grim_reprisal_aura::HandleProc, EFFECT_0, SPELL_AURA_DUMMY);
    }
};"""

@pytest.fixture
def lines():
    return input_as_string.split('\n')

@pytest.fixture
def expect() -> str:
    return expected_output

@pytest.fixture
def expect_spell_string() -> str:
    return "RegisterSpellScript(spell_yogg_saron_grim_reprisal_aura);"

def test_always_passes():
    assert True

def test_find_start_last_index(lines):
    start_index, last_index = find_start_last_index(lines)
    assert start_index == 1
    assert last_index == 37

def test_find_name_of_script(lines):
    assert find_name_of_script(lines) == "spell_yogg_saron_grim_reprisal"

def test_get_script_type(lines):
    start_index, last_index = find_start_last_index(lines)
    assert get_script_type(lines, start_index) == ScriptType.AURA

def test_find_register_start_end_index(lines):
    start_index, last_index = find_start_last_index(lines)
    register_start_index, register_end_index = find_register_start_end_index(lines, start_index)
    assert lines[register_start_index].lstrip() == "void Register() override"
    assert register_end_index == register_start_index + 3
    assert lines[register_end_index] == "        }"

def test_find_register_statements(lines):
    register_statements = find_register_statements(lines, 1)
    assert len(register_statements) == 1
    assert register_statements[0] == "            OnEffectProc += AuraEffectProcFn(spell_yogg_saron_grim_reprisal_AuraScript::HandleProc, EFFECT_0, SPELL_AURA_DUMMY);"

def test_format_register_statements():
    register_statements = ["            OnEffectProc += AuraEffectProcFn(spell_yogg_saron_grim_reprisal_AuraScript::HandleProc, EFFECT_0, SPELL_AURA_DUMMY);"]
    formatted_statements = format_register_statements(register_statements)
    assert len(formatted_statements) == 1
    assert formatted_statements[0] == "        OnEffectProc += AuraEffectProcFn(spell_yogg_saron_grim_reprisal_aura::HandleProc, EFFECT_0, SPELL_AURA_DUMMY);"

def test_find_content_start_end_index(lines):
    content_index_start, content_index_end = find_content_start_end_index(lines)
    assert lines[content_index_start].lstrip() == "PrepareAuraScript(spell_yogg_saron_grim_reprisal_AuraScript);"
    assert lines[content_index_end].lstrip() == "void Register() override"

def test_format_content_start_end_index(lines):
    content_index_start, content_index_end = find_content_start_end_index(lines)
    content_statements = lines[content_index_start+1:content_index_end]
    formatted_content_statements = format_content_statements(content_statements);
    assert len(formatted_content_statements) == 18
    assert formatted_content_statements[1] == "    bool Validate(SpellInfo const* /*spellInfo*/) override"
    assert formatted_content_statements[2] == "    {"
    assert formatted_content_statements[3] == "        return ValidateSpellInfo({ SPELL_GRIM_REPRISAL_DAMAGE });"
    assert all(f.strip() != '' or (f.strip() == '' and f == '') for f in formatted_content_statements)

def test_convert_function_block(lines, expect, expect_spell_string):
    out, spell_type, start_index, last_index, _, _ = convert_function_block(lines)
    assert spell_type == ScriptType.AURA
    assert start_index == 1
    assert last_index == 37
    assert expect == out
    out = out.split('\n')
    expect = expect.split('\n')
    assert len(expect) == len(out)
    for i in range(len(out)):
        assert out[i] == expect[i], f"{i-1}:\'{out[i-1]}\'{i}:\'{out[i]}\'\'{expect[i]}\'"




import pytest

from acore_spellscript_refactor.refactor import *

# pytest --pyargs spellscript_refactor
input_as_string =\
"""
class spell_the_eye_countercharge : public SpellScriptLoader
{
public:
    spell_the_eye_countercharge() : SpellScriptLoader("spell_the_eye_countercharge") { }

    class spell_the_eye_counterchargeScript : public AuraScript
    {
        PrepareAuraScript(spell_the_eye_counterchargeScript);

        bool PrepareProc(ProcEventInfo&  /*eventInfo*/)
        {
            // xinef: prevent charge drop
            PreventDefaultAction();
            return true;
        }

        void Register() override
        {
            DoCheckProc += AuraCheckProcFn(spell_the_eye_counterchargeScript::PrepareProc);
        }
    };

    AuraScript* GetAuraScript() const override
    {
        return new spell_the_eye_counterchargeScript();
    }
};"""

expected_output =\
"""class spell_the_eye_countercharge_aura : public AuraScript
{
    PrepareAuraScript(spell_the_eye_countercharge_aura);

    bool PrepareProc(ProcEventInfo&  /*eventInfo*/)
    {
        // xinef: prevent charge drop
        PreventDefaultAction();
        return true;
    }

    void Register() override
    {
        DoCheckProc += AuraCheckProcFn(spell_the_eye_countercharge_aura::PrepareProc);
    }
};"""


@pytest.fixture
def lines():
    return input_as_string.split('\n')

@pytest.fixture
def expect() -> str:
    return expected_output

def test_always_passes():
    assert True

def test_convert_function_block(lines, expect):
    out, _, _, _, _, _ = convert_function_block(lines)
    out = out.split('\n')
    expect = expect.split('\n')
    for i in range(len(out)):
        assert out[i] == expect[i]
    assert len(expect) == len(out)
    assert expect == out




# connect with database
import re

import mysql.connector

from .colors import Color, color
from .logger import logger

HOST="localhost"
USER="acore"
PASSWORD="acore"
DATABASE="acore_world"

def format_spell_ids(spell_ids):
    logger.debug(f'{spell_ids=}')
    spell_ids_format = ','.join(str(spell_id) for spell_id in spell_ids)
    logger.debug(f'{spell_ids_format=}')
    spell_ids_format = '('+spell_ids_format+')'
    logger.debug(f'{spell_ids_format=}')
    return spell_ids_format

def generate_sql_update_script_name(script_name_original: str, script_name: str) -> str:
    try:
        spell_ids = db_lookup_ids(script_name_original)
    except mysql.connector.Error as error:
        logger.error(color(f"{error=}", Color.RED))
        logger.error(color(f"failed for {script_name=}. Incorrect database credentials", Color.RED))
        sql_update_script_name = """UPDATE `spell_script_names` SET `ScriptName`='{}' WHERE `spell_id`=XXXXX AND `ScriptName`='{}';\n""".format(script_name, script_name_original)
        return sql_update_script_name

    if len(spell_ids) == 0:
        logger.error(color(f'no spell_ids found for {script_name=}', Color.RED))
        return ''
    elif len(spell_ids) == 1:
        spell_id = spell_ids[0]
        sql_update_script_name = """UPDATE `spell_script_names` SET `ScriptName`='{}' WHERE `spell_id`={} AND `ScriptName`='{}';\n""".format(script_name, spell_id, script_name_original)
        return sql_update_script_name
    else:
        spell_ids_format = format_spell_ids(spell_ids)
        sql_update_script_name = """UPDATE `spell_script_names` SET `ScriptName`='{}' WHERE `spell_id` IN {} AND `ScriptName`='{}';\n""".format(script_name, spell_ids_format, script_name_original)
        return sql_update_script_name

def db_lookup_ids(script_name: str) -> list[int] :
    my_db = mysql.connector.connect(
        host=HOST,
        user=USER,
        password=PASSWORD,
        database=DATABASE
    )
    my_cursor = my_db.cursor()
    my_cursor.execute(f"SELECT `spell_id` FROM `spell_script_names` WHERE `ScriptName` = '{script_name}'")
    result = my_cursor.fetchall()
    ids = [x[0] for x in result]
    return ids


import argparse

from .refactor import format_first_block_in_file
from .util.logger import logger, set_debug

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('source_file', type=str)
    parser.add_argument('dest_file', nargs='?', type=str, default=None)
    parser.add_argument('--skip', type=int, default=0, help="amount of lines to skip")
    parser.add_argument('-v', '--verbose', action='store_true')
    parser.add_argument('--commit', action='store_true')
    parser.add_argument('--sql_file', type=str, default='script_name_updates.sql')
    args = parser.parse_args()
    if args.verbose:
        set_debug()
    if not args.dest_file:
        args.dest_file = args.source_file
    format_first_block_in_file(args.source_file, args.dest_file, skip=args.skip, sql_path=args.sql_file, create_commit=args.commit)

if __name__ == '__main__':
    main()


import pytest
import os

from acore_spellscript_refactor.refactor import *

ENV_GITHUB_CI = "CI"

FILENAME_READ = "tests/test_with_input_file_with_args/boss_hadronox_read.cpp"
FILENAME_WRITE = "tests/test_with_input_file_with_args/boss_hadronox_write.cpp"
FILENAME_EXPECTED = "tests/test_with_input_file_with_args/boss_hadronox_expected.cpp"

@pytest.fixture
def expect():
    with open(FILENAME_EXPECTED, 'r') as file:
        lines = file.read();
        return lines.split('\n')

def test_always_passes():
    assert True

def test_format_first_block_in_file(expect):
    format_first_block_in_file(FILENAME_READ, FILENAME_WRITE)

    with open(FILENAME_WRITE, 'r') as file:
        lines = file.read();
        lines = lines.split('\n')

    for i in range(len(lines)):
        assert lines[i] == expect[i], f"\'{i}:\'{lines[i]}\'\'{expect[i]}\'"
    assert len(lines) == len(expect)

def test_format_content_statement():
    input = 'spell_hadronox_summon_periodic_aura(const char* name, uint32 delay, uint32 spellEntry) : SpellScriptLoader(name), _delay(delay), _spellEntry(spellEntry) { }'
    expected = 'spell_hadronox_summon_periodic_aura(uint32 delay, uint32 spellEntry) : _delay(delay), _spellEntry(spellEntry) { }'
    out = format_content_statement(input)
    assert out == expected



import logging


class Logger():
    _logger = None
    def __init__(self, level=logging.INFO):
        logger = logging.getLogger(__name__)
        logger.setLevel(level)

        # Create a formatter to define the log format
        format = "%(levelname)s:%(filename)s:%(lineno)03d:%(funcName)20s():%(message)s"
        # format = "%(levelname)s:%(filename)s:%(funcName)s():%(message)s"
        formatter = logging.Formatter(format)

        # Create a stream handler to print logs to the console
        console_handler = logging.StreamHandler()
        console_handler.setLevel(level)  # You can set the desired log level for console output
        console_handler.setFormatter(formatter)

        # Add the handler to the logger
        logger.addHandler(console_handler)
        self._logger = logger

    @property
    def logger(self):
        return self._logger


logger = Logger().logger

def set_debug():
    global logger
    logger = Logger(level=logging.DEBUG)



import pytest

from acore_spellscript_refactor.refactor import *

def test_always_passes():
    assert True

@pytest.mark.parametrize('line, spells',  [
    ('me->CastSpell(me, RAID_MODE(SPELL_TRAITOR_KING_10, SPELL_TRAITOR_KING_25, SPELL_TRAITOR_KING_10, SPELL_TRAITOR_KING_25), true);', ['SPELL_TRAITOR_KING_10', 'SPELL_TRAITOR_KING_25']),
    ('t->CastSpell(t, SPELL_GRAB_TRIGGERED, true);', [ 'SPELL_GRAB_TRIGGERED']),
    ('t->CastSpell(t, SPELL_GRAB_TRIGGERED);', [ 'SPELL_GRAB_TRIGGERED']),
    ('t->CastSpell(t, 6969, true);', [ '6969']),
    ('t->CastSpell(t, 7171);', [ '7171']),
    ('me->CastCustomSpell(SPELL_ACTIVATE_CONSTRUCT, SPELLVALUE_MAX_TARGETS, 1, (Unit*)nullptr, false);', [ 'SPELL_ACTIVATE_CONSTRUCT']),
    ('GetUnitOwner()->ApplySpellImmune(SPELL_DEATH_RAY_DAMAGE_REAL, IMMUNITY_ID, SPELL_DEATH_RAY_DAMAGE_REAL, true);', [ 'SPELL_DEATH_RAY_DAMAGE_REAL']),
    ('GetUnitOwner()->ApplySpellImmune(SPELL_DEATH_RAY_DAMAGE_REAL, IMMUNITY_ID, SPELL_DEATH_RAY_DAMAGE_REAL, true);', [ 'SPELL_DEATH_RAY_DAMAGE_REAL']),
    ('target->CastCustomSpell(dmgInfo.GetAttacker(), SPELL_REFLECTIVE_SHIELD_T, &bp, nullptr, nullptr, true, nullptr, aurEff);', [ 'SPELL_REFLECTIVE_SHIELD_T']),
    ('target->CastCustomSpell(dmgInfo.GetAttacker(), 717878, &bp, nullptr, nullptr, true, nullptr, aurEff);', [ '717878']),
    ('target->CastCustomSpell(dmgInfo.G7etAttacker(), 717878, &2bp, nullptr, nullptr, true, nullptr, aurEff);', [ '717878']),
    ('GetUnitOwner()->CastSpell(target, GetSpellInfo()->Effects[effect->GetEffIndex()].TriggerSpell, true);', []),
    ('GetCaster()->CastSpell(GetHitUnit(), (*i)->GetAmount(), true);', []),
    ('GetCaster()->CastSpell(target, GetEffectValue(), true);', []),
    ('GetCaster()->CastSpell(target, GetEffectValue(), true);', []),
    ('GetUnitOwner()->CastSpell(GetUnitOwner(), RAND(SPELL_SINFUL_PERIODIC, SPELL_SINISTER_PERIODIC, SPELL_VILE_PERIODIC, SPELL_WICKED_PERIODIC), true);', ['SPELL_SINFUL_PERIODIC', 'SPELL_SINISTER_PERIODIC', 'SPELL_VILE_PERIODIC', 'SPELL_WICKED_PERIODIC']),
    ('(player->GetAuraCount(SPELL_OSCILLATION_FIELD) == 5 && player->GetQuestStatus(QUEST_GAUGING_THE_RESONANT_FREQUENCY) == QUEST_STATUS_INCOMPLETE)', ['SPELL_OSCILLATION_FIELD']),
])

def test_find_spell_to_validate(line, spells):
    found_spells = find_spells_to_validate(line)
    assert all(f in spells for f in found_spells)
    assert len(found_spells) == len(spells)

def test_format_validate_function():
    spells = ['SPELL_GRAB_TRIGGERED']
    expected = """
    bool Validate(SpellInfo const* /*spellInfo*/) override
    {
        return ValidateSpellInfo({ SPELL_GRAB_TRIGGERED });
    }\n"""
    out = format_validate_function(spells)
    assert out == expected



import pytest

from acore_spellscript_refactor.refactor import *

def test_always_passes():
    assert True

@pytest.mark.parametrize('spell_script, spell_script_without_suffix',  [
    ('spell_aura_of_despair', 'spell_aura_of_despair'),
    ('spell_aura_of_despair_aura', 'spell_aura_of_despair'),
    ('spell_aura_of_despair_spell', 'spell_aura_of_despair'),
])

def test_remove_aura_spell_suffix(spell_script, spell_script_without_suffix):
    assert remove_aura_spell_suffix(spell_script) == spell_script_without_suffix

def test_replace_new_with_RegisterSpellScript_aura_of_despair():
    lines_input    = ['void AddSC_', '    new spell_aura_of_despair();']
    lines_expected = ['void AddSC_', '    RegisterSpellScript(spell_aura_of_despair);']
    lines, _ = replace_new_with_RegisterSpellScript(lines_input, 'spell_aura_of_despair', ScriptType.AURA)
    assert len(lines) == len(lines_expected)
    assert lines[1] == lines_expected[1]


def test_replace_new_with_RegisterSpellScript_hadronox():
    lines_input = [
        'void AddSC_',
        '    new spell_hadronox_summon_periodic("spell_hadronox_summon_periodic_necromancer", 10000, SPELL_SUMMON_ANUBAR_NECROMANCER);',
        '    new spell_hadronox_summon_periodic("spell_hadronox_summon_periodic_crypt_fiend", 5000, SPELL_SUMMON_ANUBAR_CRYPT_FIEND);'
    ]
    lines_expected = [
        'void AddSC_',
        '    RegisterSpellScriptWithArgs(spell_hadronox_summon_periodic_aura, "spell_hadronox_summon_periodic_necromancer_aura", 10000, SPELL_SUMMON_ANUBAR_NECROMANCER);',
        '    RegisterSpellScriptWithArgs(spell_hadronox_summon_periodic_aura, "spell_hadronox_summon_periodic_crypt_fiend_aura", 5000, SPELL_SUMMON_ANUBAR_CRYPT_FIEND);'
    ]

    lines, _ = replace_new_with_RegisterSpellScript(lines_input, 'spell_hadronox_summon_periodic_aura', ScriptType.AURA)
    assert lines[1] == lines_expected[1]
    assert lines[2] == lines_expected[2]
    assert len(lines) == len(lines_expected)


import pytest

from acore_spellscript_refactor.refactor import *

FILENAME_READ = "tests/test_pair_with_validate/boss_ignis_read.cpp"
FILENAME_WRITE = "tests/test_pair_with_validate/boss_ignis_write.cpp"
FILENAME_EXPECTED = "tests/test_pair_with_validate/boss_ignis_expected.cpp"

@pytest.fixture
def expect():
    with open(FILENAME_EXPECTED, 'r') as file:
        lines = file.read();
        return lines.split('\n')

def test_always_passes():
    assert True

def test_format_spells():
   input = ['SPELL_ABC', '1234', 'SPELL_TEST']
   output = format_spells(input)
   expected = 'SPELL_ABC, 1234, SPELL_TEST'
   assert output == expected

def test_format_first_block_in_file(expect):
    format_first_block_in_file(FILENAME_READ, FILENAME_WRITE, skip=450)

    with open(FILENAME_WRITE, 'r') as file:
        lines = file.read();
        lines = lines.split('\n')

    assert len(lines) == len(expect)
    for i in range(len(lines)):
        assert lines[i] == expect[i], f"\'{i}:\'{lines[i]}\'\'{expect[i]}\'"



import argparse
from pathlib import Path

from .refactor import format_first_block_in_file
from .util.logger import logger, set_debug

def discover():
    parser = argparse.ArgumentParser()
    parser.add_argument('--skip', type=int, default=0, help="amount of lines to skip")
    parser.add_argument('-v', '--verbose', action='store_true')
    parser.add_argument('--commit', action='store_true')
    parser.add_argument('--recursive', action='store_true')
    parser.add_argument('--sql_file', type=str, default='script_name_updates.sql')
    args = parser.parse_args()
    if args.verbose:
        set_debug()
    cwd = Path(".")
    if args.recursive:
        cpp_files = cwd.glob("**/*cpp")
    else:
        cpp_files = cwd.glob("*cpp")
    for cpp in list(cpp_files):
        source_file = cpp.absolute()
        dest_file = source_file
        for _ in range(100):
            try:
                format_first_block_in_file(source_file, dest_file, skip=0, sql_path=args.sql_file, create_commit=args.commit)
            except ValueError:
                break

if __name__ == '__main__':
    discover()


import re
from enum import Enum
from typing import Union
from pathlib import Path

from .util.colors import Color, color
from .util.logger import logger
from .util.db import generate_sql_update_script_name

class ScriptType(Enum):
    AURA = 1
    SPELL = 2
    PAIR = 3

def find_start_last_index(lines_to_search):
    start_index = None
    for i, line in enumerate(lines_to_search):
        if (start_index is None and re.match("class .* : public SpellScriptLoader", line)):
            start_index = i
        if (start_index is None and re.match("class .* : SpellScriptLoader", line)):
            start_index = i
        if start_index:
            logger.debug(f"{i:,=} {line=}")
            if line.rstrip() == '};':
                return start_index, i
    logger.error(color(f"No spells scripts found", Color.RED))
    raise ValueError

def find_name_of_script(lines_to_search, start_index=0):
    count = 0
    class_name_found = False
    loader_name = None
    for line in lines_to_search[start_index:]:
        logger.debug(f"{line=}")
        if ': SpellScriptLoader(' not in line and 'class ' in line and ("public SpellScriptLoader" in line or ": SpellScriptLoader" in line):
            class_name = re.findall(r'class ([\w_]+)', line)[0]
            logger.debug(f"{class_name=}")
            class_name_found = True
        if ': SpellScriptLoader(' in line:
            loader_name_found = re.findall(r'SpellScriptLoader\("([\w_]+)"\)', line)
            if len(loader_name_found) > 0:
                logger.debug(f"{line=} loader_name not found: skipping")
                loader_name = loader_name_found[0][:]
                logger.debug(f"{loader_name=}")
        if class_name_found:
            count +=1
        if count >= 4:
            return loader_name if loader_name is not None else class_name

def get_script_type(lines_to_search, start_index=0) -> ScriptType:
    isAuraScript = None
    isSpellScript = None
    for line in lines_to_search[start_index:]:
        logger.debug(f"line=")
        if "AuraScript" in line:
            logger.debug(f"AuraScript {line=}")
            isAuraScript = True
        if "SpellScript" in line and not "SpellScriptLoader" in line:
            logger.debug(f"SpellScript {line=}")
            isSpellScript = True
        if line.rstrip() == "};":
            break
    if isAuraScript and isSpellScript:
        return ScriptType.PAIR
    elif isAuraScript:
        return ScriptType.AURA
    elif isSpellScript:
        return ScriptType.SPELL
    else:
        logger.error(color("Unable to determine type!", Color.RED))

def find_register_start_end_index(lines_to_search, start_index=0):
    start = None
    logger.debug(f"{start_index=}")
    for i, line in enumerate(lines_to_search[start_index:]):
        logger.debug(f"{i:03};{line}")
        if 'void Register' in line:
           start = i
        if start:
            if line.strip() == '}':
                absolute_start = start+start_index
                absolute_end = i+start_index
                logger.debug(f"{start:,=};{absolute_start:,=} {lines_to_search[start]}")
                logger.debug(f"{i:,=};{absolute_end:,=} {lines_to_search[i]}")
                return absolute_start, absolute_end

def find_register_statements(lines, start_index):
    register_index_start, register_index_bracket = find_register_start_end_index(lines[start_index:])
    register_index_start += start_index
    register_index_bracket += start_index
    register_statements = lines[register_index_start+2:register_index_bracket]
    logger.debug(f"{register_index_start:,=};{register_index_bracket:,=}")
    for i, dbg_out in enumerate(register_statements):
        logger.debug(f"{i:03}:{dbg_out}")
    return register_statements

def format_register_statements(register_statements, original=None, new=None):
    register_statements_format = []
    for c in register_statements:
        if c.strip() == '':
            c = ''
        else:
            c = 8*' ' + c.lstrip()
            c = c.replace('_AuraScript','_aura').replace('_aura_aura','_aura')
            c = c.replace('AuraScript','_aura').replace('_aura_aura','_aura')
            c = c.replace('_SpellScript','')
            c = c.replace('SpellScript','')
            if original is not None and new is not None:
                c = c.replace(original+'Script',new)
        register_statements_format.append(c)
    for i, dbg_out in enumerate(register_statements):
        logger.debug(f"before:{i:03}:{dbg_out}")
    for i, dbg_out in enumerate(register_statements_format):
        logger.debug(f"after :{i:03}:{dbg_out}")
    return register_statements_format

def find_content_start_end_index(lines_to_search, start_index=0, script_name=''):
    start = None
    for i, line in enumerate(lines_to_search[start_index:]):
        if any(p in line for p in ['PrepareAuraScript', 'PrepareSpellScript']):
            start = i
        if start:
            if 'void Register' in line:
                absolute_start = start+start_index
                absolute_end = i+start_index
                logger.debug(f"{absolute_start=},{absolute_end=}")
                return start+start_index, i+start_index

def find_spells_to_validate(line) -> list[str]:
    cases_to_validate = [
        '->CastCustomSpell(',
        '->CastSpell(',
        '->ApplySpellImmune',
        '->HasAura(',
        '->GetAuraCount(',
        '->RemoveAurasDueToSpell('
    ]
    if any(c in line for c in cases_to_validate ):
        logger.debug(f"{line=}")
        spells = []
        spells_found = re.findall('(SPELL_[A-Z_0-9]{,}|[0-9]{4,})', line)
        for spell in spells_found:
            logger.debug(f"{spell=}")
            if spell not in spells:
                spells.append(spell)
        return spells
    return []

def format_spells(spells):
    return ', '.join(spells)

def format_validate_function(spells) -> str:
    spells_formatted = format_spells(spells)
    out ="""
    bool Validate(SpellInfo const* /*spellInfo*/) override
    {
        return ValidateSpellInfo({ """+spells_formatted+""" });
    }\n"""
    return out

def create_validate_lines(lines_to_search, start_index=0) -> str:
    logger.debug(f"{start_index=}")
    spells = []
    for i, line in enumerate(lines_to_search[start_index:]):
        logger.debug(f"{i:03};{line}")
        spells_found = find_spells_to_validate(line)
        for spell in spells_found:
            if spell not in spells:
                spells.append(spell)
        if 'void Register' in line:
            break
    if len(spells) == 0:
        return ''
    formatted = format_validate_function(spells)
    logger.debug(f"{formatted=}")
    return formatted

def find_variables(lines, start_index=0) -> tuple[bool, int, int]:
    start = None
    end = None
    foundRegister = False
    foundEndRegister = False
    foundVariables = False
    for i, line in enumerate(lines[start_index:]):
        logger.debug(line)
        if line.rstrip() == '};':
            start = 0
            end = i
            break
        if 'void Register' in line:
            logger.debug("Found register")
            foundRegister = True
        if foundRegister and (line == 8*" "+"}" or line == 8*" "+"};"):
            start = i + 1
            foundEndRegister = True
            logger.debug("Found end register")
            logger.debug(f"{i+start_index} {line}")
        if foundEndRegister and ("protected:" in line or "public:" in line or "private:" in line):
            logger.debug("Found variable")
            logger.debug(f"{i+start_index}")
            logger.debug(line)
            foundVariables = True
        if foundRegister and (line == '    };' or line == '};'):
            end = i
            break
    # if start is None or end is None:
     #    return foundVariables, start_index, start_index
    # else:
    return foundVariables, start+start_index, end+start_index

def find_content_statements(lines, start_index):
    content_index_start, content_index_end = find_content_start_end_index(lines[start_index:])
    content_index_start += start_index
    content_index_end += start_index
    content_statements = lines[content_index_start+1:content_index_end]
    return content_statements

def format_content_statements(content_statements, original=None, new=None):
    content_statements_format = []
    for c in content_statements:
        if c.strip() == '':
            c = ''
        elif 8*' ' in c:
            c = (4*' ').join(c.split(4*' ')[1:])
        c = c.replace('_AuraScript','_aura').replace('_aura_aura','_aura')
        c = c.replace('_SpellScript','')
        if original is not None and new is not None:
            c = c.replace(original+'Script',new)
        content_statements_format.append(c)
    for i, dbg_out in enumerate(content_statements):
        logger.debug(f"before:{i:03}:{dbg_out}")
    for i, dbg_out in enumerate(content_statements_format):
        logger.debug(f"after :{i:03}:{dbg_out}")
    return content_statements_format

def format_variables(variables):
    formatted_variables = []
    for c in variables:
        if c.strip() == '':
            continue
        elif 4*' ' in c:
            c = (4*' ').join(c.split(4*' ')[1:])
        formatted_variables.append(c)
    return formatted_variables

def is_validate_in_content(content_statements):
    return any('bool Validate(' in c for c in content_statements)

def find_constructor_statement(lines_to_search, start_index) -> str:
    has_public = False
    for i, line in enumerate(lines_to_search[start_index:]):
        if any(p in line for p in ['PrepareAuraScript', 'PrepareSpellScript']):
            return ''
        if 'public:' in line.lstrip():
            has_public = True
        if has_public and ': SpellScriptLoader(name' in line:
            return line
    return ''

def format_content_statement(line):
    # input = 'spell_hadronox_summon_periodic(const char* name, uint32 delay, uint32 spellEntry) : SpellScriptLoader(name), _delay(delay), _spellEntry(spellEntry) { }'
    # expected = 'spell_hadronox_summon_periodic_aura(uint32 delay, uint32 spellEntry) : _delay(delay), _spellEntry(spellEntry) { }'
    line = line.replace('const char* name, ','')
    line = line.replace('SpellScriptLoader(name), ','')
    return line

def convert_aura_or_spell_script(lines, start_index, last_index, script_type, script_name, original_script_name) -> tuple[list[str], ScriptType, int, int, str, str]:
    type = 'SpellScript' if script_type == ScriptType.SPELL else 'AuraScript'
    prepare = 'Spell' if script_type == ScriptType.SPELL else 'Aura'
    register_statements = find_register_statements(lines, start_index)
    register_statements = format_register_statements(register_statements, original=original_script_name, new=script_name)
    content_statements = find_content_statements(lines, start_index)
    constructor_statement = find_constructor_statement(lines, start_index)
    constructor_statement = constructor_statement.replace(original_script_name, script_name)
    constructor_statement_formatted = format_content_statement(constructor_statement)
    constructor = f"\npublic:\n{constructor_statement_formatted}\n" if constructor_statement else ''
    validate = create_validate_lines(lines, start_index) if not is_validate_in_content(content_statements) else ''
    content_statements = format_content_statements(content_statements, original=original_script_name, new=script_name)

    content: str = '\n'.join(content_statements)
    register_formatted: str = '\n'.join(register_statements)
    isVariables, start, end = find_variables(lines, start_index)
    if not isVariables:
        variables = ''
    else:
        variables = 2*'\n'+'\n'.join(format_variables(lines[start:end]))
    out_formatted = \
"""class """+script_name+""" : public """+type+"""\n{
    Prepare"""+prepare+"""Script("""+script_name+");\n"+constructor+validate\
+content+\
"""
    void Register() override
    {\n"""+\
register_formatted+\
"""
    }"""+variables+"""
};"""
    for i, dbg_out in enumerate(lines[start_index:last_index+1]):
        logger.debug(f"before:{i:03}:{dbg_out}")
    for i, dbg_out in enumerate(out_formatted.split('\n')):
        logger.debug(f"after :{i:03}:{dbg_out}")

    return out_formatted, script_type, start_index, last_index, script_name, original_script_name

def find_spell_block_in_pair(lines, script_type):
    reg = None
    if script_type == ScriptType.AURA:
        reg = ".*class .*_AuraScript : public AuraScript.*"
    elif script_type == ScriptType.SPELL:
        reg = ".*class .*_SpellScript : public SpellScript.*"
    else:
        logger.error(color("Invalid type", Color.RED))

    start_index = None
    end_index = None
    for i, line in enumerate(lines):
        if (start_index is None and re.match(reg, line)):
            start_index = i
        if start_index:
            logger.debug(f"{i:,=} {line=}")
            if line.rstrip() == '    };':
                end_index = i
                break
    return lines[start_index:end_index+1]

def format_script_name(script_name, script_type):
    if script_type == ScriptType.AURA:
        return (script_name + '_aura').replace('_aura_aura','_aura')
    elif script_type == ScriptType.SPELL:
        return script_name
    elif script_type == ScriptType.PAIR:
        return script_name

def convert_function_block(lines: list[str]) -> tuple[str, ScriptType, int, int, str, str]:
    start_index, last_index = find_start_last_index(lines)
    logger.debug(f"{start_index=}{last_index=}")
    script_type = get_script_type(lines, start_index)
    logger.debug(f"{script_type=}")
    script_name = find_name_of_script(lines, start_index)
    original_script_name = script_name
    formatted_script_name = format_script_name(script_name, script_type)
    logger.debug(f"{script_name=}")
    if script_type != ScriptType.PAIR:
        return convert_aura_or_spell_script(lines, start_index, last_index, script_type, formatted_script_name, original_script_name)
    spell_script_lines = find_spell_block_in_pair(lines[start_index:last_index], ScriptType.SPELL)
    formatted_script_name = format_script_name(script_name, ScriptType.SPELL)
    converted_spell, _, _, _, _, _ = convert_aura_or_spell_script(spell_script_lines, 0, len(spell_script_lines), ScriptType.SPELL, formatted_script_name, original_script_name)

    aura_script_lines = find_spell_block_in_pair(lines[start_index:last_index], ScriptType.AURA)
    formatted_script_name = format_script_name(script_name, ScriptType.AURA)
    converted_aura, _, _, _, _, _ = convert_aura_or_spell_script(aura_script_lines, 0, len(aura_script_lines), ScriptType.AURA, formatted_script_name, original_script_name)

    return (converted_spell, converted_aura), script_type, start_index, last_index, script_name, original_script_name

ARGS_REGISTER_LINES = []

def format_RegisterSpellScript(script_name, script_type: ScriptType, args=None) -> str:
    if script_type == ScriptType.PAIR:
        register_statement = f"    RegisterSpellAndAuraScriptPair({script_name}, {script_name}_aura);"
    elif args is not None:
        if script_type == ScriptType.AURA:
            ARGS_REGISTER_LINES.append(args[0].replace('"',''))
            args[0] = args[0][:-1]+'_aura"'
        register_statement = f"    RegisterSpellScriptWithArgs({script_name}, {', '.join(args)});"
    else:
        register_statement = f"    RegisterSpellScript({script_name});"
    logger.debug(f"{register_statement=}")
    return register_statement

def remove_aura_spell_suffix(script_name):
    script_name_without_aura_suffix = re.sub('_aura$', '', script_name)
    script_name_without_aura_spell_suffix = re.sub('_spell$', '', script_name_without_aura_suffix)
    return script_name_without_aura_spell_suffix


def replace_new_with_RegisterSpellScript(lines, script_name, script_type):
    script_name_search = "new "+ remove_aura_spell_suffix(script_name)
    logger.debug(f"{script_name=}")
    logger.debug(f"{script_type=}")
    logger.debug(f"{script_name_search=}")
    logger.debug(f"{lines}")
    foundVoidSc = False
    hasArguments = False
    for i, line in enumerate(lines):
        if line.startswith("void AddSC_"):
            foundVoidSc = True
        if foundVoidSc and script_name_search in line:
            logger.debug(f"{script_name_search=}")
            logger.debug(f"{script_name=}")
            if line.endswith("();"):
                new_line = format_RegisterSpellScript(script_name, script_type)
                logger.debug(f"before:{lines[i]}")
                lines[i] = new_line
                logger.debug(f"after :{new_line}")
                return lines, None
            else:
                args_found = re.findall('new \w+\((.+)\);', line)
                args = [s.strip() for s in args_found[0].split(',')]
                new_line = format_RegisterSpellScript(script_name, script_type, args=args)
                lines[i] = new_line
                logger.debug(f"after :{new_line}")
                hasArguments = True
    if hasArguments:
        return lines, args[0] # return name
    logger.error("No register name found")


def format_first_block_in_file(path_in, path_out, skip=0, sql_path='script_name_updates.sql', create_commit=False):
    logger.info(f"{path_in=}")
    logger.info(f"{path_out=}")
    logger.info(f"{sql_path=}")
    with open(path_in, 'r') as file:
        lines = file.read();
        lines = lines.split('\n')
    logger.debug(f"{len(lines)=} {path_in=}")
    out, script_type, start_index, last_index, script_name, original_script_name = convert_function_block(lines[skip:])
    if type(out) == tuple:
        spell, aura = out
        out = spell + 2*'\n' + aura
    lines = lines[:start_index+skip] + out.split('\n') + lines[last_index+skip+1:]
    lines, arg_name = replace_new_with_RegisterSpellScript(lines, script_name, script_type)

    for i, dbg_out in enumerate(lines):
        logger.debug(f"{i:03}:{dbg_out}")
    with open(path_out, 'w') as file:
        file.write('\n'.join(lines))
        logger.info(color(f"{script_name=}", Color.GREEN))

    hasUpdateQuery = False
    if script_type != ScriptType.AURA or original_script_name == script_name:
        logger.info(color(f"Skipped query for {original_script_name=}", Color.YELLOW))
    elif script_type == ScriptType.AURA:
        with open(sql_path, 'a') as file:
            sql_update_script_name = generate_sql_update_script_name(original_script_name, script_name)
            if sql_update_script_name == '':
                logger.error(color(f"Update is empty for {original_script_name=}:{script_name=}", Color.RED))
            else:
                logger.debug(f"{sql_update_script_name=}")
                file.write(sql_update_script_name)
                hasUpdateQuery = True
                logger.info(color(f"Appended {script_name} to {Path(sql_path).name}", Color.GREEN))
        if arg_name is not None:
            with open(sql_path, 'a') as file:
                for arg_name in ARGS_REGISTER_LINES:
                    arg_name_new = arg_name+'_aura' if not arg_name.endswith('_aura') else arg_name
                    if arg_name != arg_name_new:
                        sql_update_script_name = generate_sql_update_script_name(arg_name, arg_name_new)
                        if sql_update_script_name == '':
                            logger.error(color(f"Update query is empty for {original_script_name=}:{script_name=}", Color.RED))
                        else:
                            logger.debug(f"{sql_update_script_name=}")
                            file.write(sql_update_script_name)
                            hasUpdateQuery = True
                            logger.info(color(f"Appended {arg_name_new} to {Path(sql_path).name}", Color.GREEN))

    if create_commit:
        from os import system
        script_type_name = "pair"
        if script_type == ScriptType.AURA:
            script_type_name = "aura"
        elif script_type == ScriptType.SPELL:
            script_type_name = "spell"
        system("git reset") # unstage all
        if sql_path != "script_name_updates.sql" and hasUpdateQuery:
            system(f"git add {sql_path}")
        if path_in == path_out:
            system(f"git add {path_out}")
        filename_stem = Path(path_out).stem
        message = f"\"{filename_stem} {script_type_name}:{script_name}\""
        system(f"git commit -m {message}")




import pytest

from acore_spellscript_refactor.refactor import *

FILENAME_READ = "tests/test_pair_script/pair_script_read.cpp"
FILENAME_WRITE = "tests/test_pair_script/pair_script_write.cpp"
FILENAME_EXPECTED = "tests/test_pair_script/pair_script_expected.cpp"

@pytest.fixture
def expect():
   with open(FILENAME_EXPECTED, 'r') as file:
       lines = file.read();
       return lines.split('\n')

def test_always_passes():
   assert True

@pytest.mark.skip(reason="todo")
def test_format_first_block_in_file(expect):
   format_first_block_in_file(FILENAME_READ, FILENAME_WRITE)

   with open(FILENAME_WRITE, 'r') as file:
       lines = file.read();
       lines = lines.split('\n')

   assert len(lines) == len(expect)
   for i in range(len(lines)):
       assert lines[i] == expect[i], f"\'{i}:\'{lines[i]}\'\'{expect[i]}\'"


import os
import pytest

import mysql.connector

from acore_spellscript_refactor.util.db import *

ENV_GITHUB_CI = "CI"

def test_always_passes():
    assert True

@pytest.mark.skipif(os.getenv(ENV_GITHUB_CI) is not None, reason="skip this test in CI")
def test_db_lookup_ids_empty():
    result = db_lookup_ids('this_spell_does_not_exist_12379832')
    assert len(result) == 0


@pytest.mark.skipif(os.getenv(ENV_GITHUB_CI) is not None, reason="skip this test in CI")
def test_db_lookup_ids_single_spell():
    result = db_lookup_ids('spell_gen_proc_not_self')
    assert result[0] == 70803
    assert len(result) == 1

def test_format_spell_ids():
    spell_ids_format = format_spell_ids([1, 2, 3, 4])
    format_expected = '(1,2,3,4)'
    assert spell_ids_format == format_expected

@pytest.mark.skipif(os.getenv(ENV_GITHUB_CI) is not None, reason="skip this test in CI")
def test_db_write_sql():
    script_name = 'spell_gen_proc_not_self'
    result = db_lookup_ids(script_name)
    assert result[0] == 70803
    sql = generate_sql_update_script_name(script_name, script_name)
    sql_expected ="""UPDATE `spell_script_names` SET `ScriptName`='spell_gen_proc_not_self' WHERE `spell_id`=70803 AND `ScriptName`='spell_gen_proc_not_self';\n"""
    assert sql == sql_expected

@pytest.mark.skipif(os.getenv(ENV_GITHUB_CI) is not None, reason="skip this test in CI")
def test_db_write_sql_with_original():
    script_name_original = 'spell_gen_proc_not_self'
    script_name = 'spell_gen_proc_not_self_aura'
    result = db_lookup_ids(script_name_original)
    assert result[0] == 70803
    sql = generate_sql_update_script_name(script_name_original, script_name)
    sql_expected ="""UPDATE `spell_script_names` SET `ScriptName`='spell_gen_proc_not_self_aura' WHERE `spell_id`=70803 AND `ScriptName`='spell_gen_proc_not_self';\n"""
    assert sql == sql_expected

@pytest.mark.skipif(os.getenv(ENV_GITHUB_CI) is not None, reason="skip this test in CI")
def test_db_write_sql_fail():
    script_name = 'doesnotexist123823'
    result = db_lookup_ids(script_name)
    assert len(result) == 0
    sql = generate_sql_update_script_name(script_name, script_name)
    sql_expected = ''
    assert sql == sql_expected

def test_failed_connection():
    try:
        mysql.connector.connect(
            host="localhost",
            user="acore",
            password="wrongpassword",
            database="acore_world"
        )
    except mysql.connector.Error:
        assert True
        return
    assert False